@@ -75,20 +75,28 @@ namespace Accounts {
 class Account::Private
 {
 public:
-    Private()
+    Private():
+        m_account(0),
+        m_cancellable(g_cancellable_new())
     {
-        m_account = 0;
     }
 
-    ~Private() {}
+    ~Private()
+    {
+        g_cancellable_cancel(m_cancellable);
+        g_object_unref(m_cancellable);
+        m_cancellable = NULL;
+    }
 
     AgAccount *m_account;  //real account
+    GCancellable *m_cancellable;
     QString prefix;
 
     static void on_display_name_changed(Account *self);
     static void on_enabled(Account *self, const gchar *service_name,
                            gboolean enabled);
-    static void account_store_cb(AgAccount *account, const GError *error,
+    static void account_store_cb(AgAccount *account,
+                                 GAsyncResult *res,
                                  Account *self);
     static void on_deleted(Account *self);
 };
@@ -374,12 +382,12 @@ QStringList Account::allKeys() const
     QStringList allKeys;
     AgAccountSettingIter iter;
     const gchar *key;
-    const GValue *val;
+    GVariant *val;
 
     /* iterate the settings */
     QByteArray tmp = d->prefix.toLatin1();
     ag_account_settings_iter_init(d->m_account, &iter, tmp.constData());
-    while (ag_account_settings_iter_next(&iter, &key, &val))
+    while (ag_account_settings_iter_get_next(&iter, &key, &val))
     {
         allKeys.append(QString(ASCII(key)));
     }
@@ -516,7 +524,7 @@ void Account::remove(const QString &key)
     {
         QString full_key = d->prefix + key;
         QByteArray tmpkey = full_key.toLatin1();
-        ag_account_set_value(d->m_account, tmpkey.constData(), NULL);
+        ag_account_set_variant(d->m_account, tmpkey.constData(), NULL);
     }
 }
 
@@ -530,30 +538,69 @@ void Account::remove(const QString &key)
 void Account::setValue(const QString &key, const QVariant &value)
 {
     TRACE();
-    GValue val= {0, {{0}}};
 
-    if (!variantToGValue(value, &val)) {
+    GVariant *variant = qVariantToGVariant(value);
+    if (variant == 0) {
         return;
     }
 
     QString full_key = d->prefix + key;
     QByteArray tmpkey = full_key.toLatin1();
-    ag_account_set_value(d->m_account, tmpkey.constData(), &val);
-    g_value_unset(&val);
+    ag_account_set_variant(d->m_account, tmpkey.constData(), variant);
 }
 
-void Account::Private::account_store_cb(AgAccount *account, const GError *err,
+void Account::Private::account_store_cb(AgAccount *account,
+                                        GAsyncResult *res,
                                         Account *self)
 {
     TRACE() << "Saved accunt ID:" << account->id;
 
-    if (err) {
-        emit self->error(Error(err));
+    GError *error = NULL;
+    ag_account_store_finish(account, res, &error);
+    if (error) {
+        if (error->domain == G_IO_ERROR &&
+            error->code == G_IO_ERROR_CANCELLED) {
+            TRACE() << "Account destroyed, operation cancelled";
+        } else {
+            emit self->error(Error(error));
+        }
+        g_error_free(error);
     } else {
         emit self->synced();
     }
+}
 
-    Q_UNUSED(account);
+/*!
+ * Retrieves the value of an account setting, as a QVariant.
+ * @param key The key whose value must be retrieved.
+ * @param defaultValue Value returned if the key is unset.
+ * @param source Indicates whether the value comes from the account, the
+ * service template or was unset.
+ * @see valueAsString
+ * @see valueAsInt
+ * @see valueAsBool
+ *
+ * @return The value associated to \a key.
+ *
+ * This method operates on the currently selected service.
+ */
+QVariant Account::value(const QString &key, const QVariant &defaultValue,
+                        SettingSource *source) const
+{
+    QString full_key = d->prefix + key;
+    QByteArray ba = full_key.toLatin1();
+    AgSettingSource settingSource;
+    GVariant *variant =
+        ag_account_get_variant(d->m_account, ba.constData(), &settingSource);
+    if (source != 0) {
+        switch (settingSource) {
+        case AG_SETTING_SOURCE_ACCOUNT: *source = ACCOUNT; break;
+        case AG_SETTING_SOURCE_PROFILE: *source = TEMPLATE; break;
+        default: *source = NONE; break;
+        }
+    }
+
+    return (variant != 0) ? gVariantToQVariant(variant) : defaultValue;
 }
 
 /*!
@@ -568,49 +615,21 @@ void Account::Private::account_store_cb(AgAccount *account, const GError *err,
  * or was unset.
  *
  * This method operates on the currently selected service.
+ * @deprecated Use value(const QString &key, const QVariant &defaultValue,
+ * SettingSource *source) const instead.
  */
 SettingSource Account::value(const QString &key, QVariant &value) const
 {
-    GType type;
-
-    switch (value.type())
-    {
-    case QVariant::String:
-        type = G_TYPE_STRING;
-        break;
-    case QVariant::Int:
-        type = G_TYPE_INT;
-        break;
-    case QVariant::UInt:
-        type = G_TYPE_UINT;
-        break;
-    case QVariant::LongLong:
-        type = G_TYPE_INT64;
-        break;
-    case QVariant::ULongLong:
-        type = G_TYPE_UINT64;
-        break;
-    case QVariant::Bool:
-        type = G_TYPE_BOOLEAN;
-        break;
-    default:
-        qWarning("Unsupported type %s", value.typeName());
-        return NONE;
+    SettingSource source;
+    QVariant variant = this->value(key, QVariant(), &source);
+    if (variant.isValid()) {
+        if (value.type() != variant.type()) {
+            if (!variant.convert(value.type())) source = NONE;
+        }
+        value = variant;
     }
 
-    GValue val= {0, {{0}}};
-    g_value_init(&val, type);
-    QString full_key = d->prefix + key;
-    AgSettingSource source =
-        ag_account_get_value(d->m_account,
-                             full_key.toLatin1().constData(), &val);
-    if (source == AG_SETTING_SOURCE_NONE)
-        return NONE;
-
-    value = gvalueToVariant(&val);
-    g_value_unset(&val);
-
-    return (source == AG_SETTING_SOURCE_ACCOUNT) ? ACCOUNT : TEMPLATE;
+    return source;
 }
 
 /*!
@@ -756,9 +775,10 @@ void Account::sync()
 {
     TRACE();
 
-    ag_account_store(d->m_account,
-                     (AgAccountStoreCb)&Private::account_store_cb,
-                     this);
+    ag_account_store_async(d->m_account,
+                           d->m_cancellable,
+                           (GAsyncReadyCallback)&Private::account_store_cb,
+                           this);
 }
 
 /*!
