@@ -319,8 +319,12 @@ void AccountsTest::accountValueTestCase()
     account->setValue(QString("test"),QString("value"));
     int int_value = 666;
     account->setValue("testint", int_value);
+    QStringList names;
+    names << "Tom" << "Dick" << "Harry";
+    account->setValue("names", names);
     account->sync();
 
+    QTest::qWait(10);
     QVERIFY(m_stored);
 
     /* check that the values we wrote are retrieved successfully */
@@ -339,6 +343,8 @@ void AccountsTest::accountValueTestCase()
 
     QVERIFY(source == ACCOUNT);
 
+    QCOMPARE(account->value("names").toStringList(), names);
+
     /* test the convenience methods */
     QString strval = account->valueAsString("test");
     QCOMPARE (strval, QString("value"));
@@ -382,6 +388,7 @@ void AccountsTest::accountSyncTestCase()
                       this,  SLOT(stored()));
 
     account->sync();
+    QTest::qWait(10);
     QVERIFY(m_stored);
 
     delete account;
@@ -964,10 +971,11 @@ void AccountsTest::authDataTest()
     globalParameters["server"] = UTF8("myserver.com");
     globalParameters["port"] = 8080;
     globalParameters["other"] = UTF8("overriden parameter");
+    globalParameters["scopes"] = QStringList() << "read" << "write" << "edit";
 
     QVariantMap serviceParameters;
-    globalParameters["other"] = UTF8("better parameter");
-    globalParameters["boolean"] = true;
+    serviceParameters["other"] = UTF8("better parameter");
+    serviceParameters["boolean"] = true;
 
     account->setCredentialsId(credentialsId);
     account->setValue("auth/method", method);
